﻿within Prosjektoppgave;
package MasterAppendix

  package Components
    package Silicon

      record SiliconParameters
        extends Modelica.Icons.Record;
        parameter Modelica.SIunits.Density rho = 3200 "Density";
        parameter Modelica.SIunits.SpecificHeatCapacity cp_gliding = 812
          "Specific heat capacity when not solidifying";
        parameter Modelica.SIunits.SpecificEnthalpy h_latent = 1100e3
          "latent heat released when solidifying";
        parameter Modelica.SIunits.Temperature T_solidus = 1205+273.15
          "End point of solidification";
        parameter Modelica.SIunits.Temperature T_liquidus = 1350+273.15
          "Start point of solidification";
        parameter Modelica.SIunits.ThermalConductivity k = 23.5
          "Thermal conductivity of solid phase";
        parameter Modelica.SIunits.ThermalConductivity k_liq = 44
          "Thermal conductivity before solidification";

        annotation (Icon(coordinateSystem(preserveAspectRatio=false)), Diagram(
              coordinateSystem(preserveAspectRatio=false)));
      end SiliconParameters;

      model HeatCapSilicon

      public
        parameter Silicon.SiliconParameters    s = Silicon.SiliconParameters();
        parameter Modelica.SIunits.Temperature T_new "Temperature when reinitializing the element";
        parameter Modelica.SIunits.Mass        m "mass of capacitance";

        Modelica.SIunits.SpecificHeatCapacity  cp "Heat capacity of element (= cp*m)";
        Modelica.SIunits.Temperature           T(start=293.15, displayUnit="degC") "Temperature of element";
      //   Modelica.SIunits.TemperatureSlope      der_T(start=0) "Time derivative of temperature (= der(T))";
         Modelica.SIunits.HeatCapacity          C "heat capacity of element";

        Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a port annotation (Placement(transformation(
              origin={0,-100},
              extent={{-10,-10},{10,10}},
              rotation=90)));
        Modelica.Blocks.Interfaces.BooleanInput reStart annotation (Placement(
              transformation(extent={{-100,40},{-60,80}}), iconTransformation(extent={
                  {-100,40},{-60,80}})));

      equation
        when reStart then
          reinit(T, T_new);
        end when;

        cp =specificHeat(T, s);
        C = m*cp;
        T = port.T;
      //  der_T = der(T);
      //  m*der(T) = port.Q_flow/(if noEvent(T>s.T_liquidus) then s.cp_gliding else (if noEvent(T>=s.T_solidus) then s.cp_gliding+s.h_latent/(s.T_liquidus-s.T_solidus) else s.cp_gliding));
        C*der(T) = port.Q_flow;
        annotation (
          Icon(coordinateSystem(preserveAspectRatio=true, extent={{-100,-100},{
                  100,100}}), graphics={
              Text(
                extent={{-150,110},{150,70}},
                textString="%name",
                lineColor={0,0,255}),
              Polygon(
                points={{0,67},{-20,63},{-40,57},{-52,43},{-58,35},{-68,25},{-72,
                    13},{-76,-1},{-78,-15},{-76,-31},{-76,-43},{-76,-53},{-70,-65},
                    {-64,-73},{-48,-77},{-30,-83},{-18,-83},{-2,-85},{8,-89},{22,
                    -89},{32,-87},{42,-81},{54,-75},{56,-73},{66,-61},{68,-53},{
                    70,-51},{72,-35},{76,-21},{78,-13},{78,3},{74,15},{66,25},{54,
                    33},{44,41},{36,57},{26,65},{0,67}},
                lineColor={160,160,164},
                fillColor={192,192,192},
                fillPattern=FillPattern.Solid),
              Polygon(
                points={{-58,35},{-68,25},{-72,13},{-76,-1},{-78,-15},{-76,-31},{
                    -76,-43},{-76,-53},{-70,-65},{-64,-73},{-48,-77},{-30,-83},{-18,
                    -83},{-2,-85},{8,-89},{22,-89},{32,-87},{42,-81},{54,-75},{42,
                    -77},{40,-77},{30,-79},{20,-81},{18,-81},{10,-81},{2,-77},{-12,
                    -73},{-22,-73},{-30,-71},{-40,-65},{-50,-55},{-56,-43},{-58,-35},
                    {-58,-25},{-60,-13},{-60,-5},{-60,7},{-58,17},{-56,19},{-52,
                    27},{-48,35},{-44,45},{-40,57},{-58,35}},
                lineColor={0,0,0},
                fillColor={160,160,164},
                fillPattern=FillPattern.Solid),
              Text(
                extent={{-69,7},{71,-24}},
                lineColor={0,0,0},
                textString="%C")}),
          Diagram(coordinateSystem(preserveAspectRatio=true, extent={{-100,-100},
                  {100,100}}), graphics={
              Polygon(
                points={{0,67},{-20,63},{-40,57},{-52,43},{-58,35},{-68,25},{-72,
                    13},{-76,-1},{-78,-15},{-76,-31},{-76,-43},{-76,-53},{-70,-65},
                    {-64,-73},{-48,-77},{-30,-83},{-18,-83},{-2,-85},{8,-89},{22,
                    -89},{32,-87},{42,-81},{54,-75},{56,-73},{66,-61},{68,-53},{
                    70,-51},{72,-35},{76,-21},{78,-13},{78,3},{74,15},{66,25},{54,
                    33},{44,41},{36,57},{26,65},{0,67}},
                lineColor={160,160,164},
                fillColor={192,192,192},
                fillPattern=FillPattern.Solid),
              Polygon(
                points={{-58,35},{-68,25},{-72,13},{-76,-1},{-78,-15},{-76,-31},{
                    -76,-43},{-76,-53},{-70,-65},{-64,-73},{-48,-77},{-30,-83},{-18,
                    -83},{-2,-85},{8,-89},{22,-89},{32,-87},{42,-81},{54,-75},{42,
                    -77},{40,-77},{30,-79},{20,-81},{18,-81},{10,-81},{2,-77},{-12,
                    -73},{-22,-73},{-30,-71},{-40,-65},{-50,-55},{-56,-43},{-58,-35},
                    {-58,-25},{-60,-13},{-60,-5},{-60,7},{-58,17},{-56,19},{-52,
                    27},{-48,35},{-44,45},{-40,57},{-58,35}},
                lineColor={0,0,0},
                fillColor={160,160,164},
                fillPattern=FillPattern.Solid),
              Ellipse(
                extent={{-6,-1},{6,-12}},
                lineColor={255,0,0},
                fillColor={191,0,0},
                fillPattern=FillPattern.Solid),
              Text(
                extent={{11,13},{50,-25}},
                lineColor={0,0,0},
                textString="T"),
              Line(points={{0,-12},{0,-96}}, color={255,0,0})}));
      end HeatCapSilicon;

      model Mould

      protected
        parameter Modelica.SIunits.ThermalConductance      G = s.k*A/(d/N);

      public
        parameter Silicon.SiliconParameters                s = Silicon.SiliconParameters();
        parameter Modelica.SIunits.Temperature             T0 = 1450+273.15 "Initial temperature of the element";
        parameter Modelica.SIunits.Mass                    m = 1000 "mass of silicon";
        parameter Integer                                  N(min=1) "Number of discrete capacitances";
        parameter Modelica.SIunits.Area                    A "surface area of mould";
        final parameter Modelica.SIunits.Length            d = m/(s.rho*A) "depth of mould";

        Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a port_a
          annotation (Placement(transformation(extent={{-10,-110},{10,-90}}),
              iconTransformation(extent={{-10,-110},{10,-90}})));
        Modelica.Thermal.HeatTransfer.Components.ThermalConductor thermalConductor[N - 1](
                                                                                        each G=G)
          annotation (Placement(transformation(
              extent={{-10,-10},{10,10}},
              rotation=270,
              origin={0,12})));

        Modelica.Blocks.Interfaces.BooleanInput reStart annotation (Placement(
              transformation(extent={{-94,38},{-54,78}}), iconTransformation(extent={{
                  -94,38},{-54,78}})));
        HeatCapSilicon heatCapSilicon[N](
          each m=m/N,
          each T_new=T0,
          each s=s,
          T(each start=T0, each fixed=true))
          annotation (Placement(transformation(extent={{-10,44},{10,64}})));
      equation
        //connecting thermal ports
        for i in 1:N-1 loop
          connect(heatCapSilicon[i].port, thermalConductor[i].port_a);
          connect(thermalConductor[i].port_b, heatCapSilicon[i+1].port);
        end for;
        connect(heatCapSilicon[N].port,port_a);

        //connecting logical ports governing reinitialization
        for i in 1:N loop
          connect(reStart,heatCapSilicon[i].reStart);
        end for;

        annotation (Icon(coordinateSystem(preserveAspectRatio=false), graphics={
              Text(
                extent={{-150,110},{150,70}},
                textString="%name",
                lineColor={0,0,255}),
              Polygon(
                points={{0,67},{-20,63},{-40,57},{-52,43},{-58,35},{-68,25},{-72,
                    13},{-76,-1},{-78,-15},{-76,-31},{-76,-43},{-76,-53},{-70,-65},
                    {-64,-73},{-48,-77},{-30,-83},{-18,-83},{-2,-85},{8,-89},{22,
                    -89},{32,-87},{42,-81},{54,-75},{56,-73},{66,-61},{68,-53},{
                    70,-51},{72,-35},{76,-21},{78,-13},{78,3},{74,15},{66,25},{54,
                    33},{44,41},{36,57},{26,65},{0,67}},
                lineColor={160,160,164},
                fillColor={192,192,192},
                fillPattern=FillPattern.Solid),
              Polygon(
                points={{-58,35},{-68,25},{-72,13},{-76,-1},{-78,-15},{-76,-31},{
                    -76,-43},{-76,-53},{-70,-65},{-64,-73},{-48,-77},{-30,-83},{-18,
                    -83},{-2,-85},{8,-89},{22,-89},{32,-87},{42,-81},{54,-75},{42,
                    -77},{40,-77},{30,-79},{20,-81},{18,-81},{10,-81},{2,-77},{-12,
                    -73},{-22,-73},{-30,-71},{-40,-65},{-50,-55},{-56,-43},{-58,-35},
                    {-58,-25},{-60,-13},{-60,-5},{-60,7},{-58,17},{-56,19},{-52,
                    27},{-48,35},{-44,45},{-40,57},{-58,35}},
                lineColor={0,0,0},
                fillColor={160,160,164},
                fillPattern=FillPattern.Solid),
              Line(points={{-54,40},{46,40}}, color={255,255,255}),
              Line(points={{-76,0},{78,0}}, color={255,255,255}),
              Line(points={{-76,-40},{72,-40}}, color={255,255,255}),
              Line(points={{-40,-80},{44,-80}}, color={255,255,255}),
              Text(
                extent={{-69,7},{71,-24}},
                lineColor={0,0,0},
                textString="%C")}),                                    Diagram(
              coordinateSystem(preserveAspectRatio=false)));
      end Mould;

      model ThermalConductor
        "Thermal conductor with variable conductanceLumped thermal element transporting heat without storing it"

        parameter SiliconParameters s = SiliconParameters();
        extends Modelica.Thermal.HeatTransfer.Interfaces.Element1D;
        parameter Modelica.SIunits.Area A;
        parameter Modelica.SIunits.Length L;

        Modelica.SIunits.ThermalConductance G;
        Modelica.SIunits.ThermalConductivity k;
        Modelica.SIunits.Temperature T_avg;

      equation
        T_avg = (port_a.T+port_b.T)/2;

        k = if T_avg>s.T_solidus then s.k_liq else  s.k;

        G = k*A/L;
        Q_flow = G*dT;

        annotation (Icon(graphics={
              Text(
                extent={{-150,115},{150,75}},
                textString="%name",
                lineColor={0,0,255}),
              Rectangle(
                extent={{-90,70},{90,-70}},
                lineColor={0,0,0},
                pattern=LinePattern.None,
                fillColor={192,192,192},
                fillPattern=FillPattern.Backward)}));
      end ThermalConductor;

      function specificHeat
        "Approximation of the specific heat of silicon, including the enthalpy of fusion"
        input Modelica.SIunits.Temperature            T;
        input SiliconParameters s;
        output Modelica.SIunits.SpecificHeatCapacity  cp;

      algorithm

        if T<s.T_solidus then
          cp := s.cp_gliding;
        elseif T>=s.T_solidus and T<=s.T_liquidus then
          cp := s.h_latent/(s.T_liquidus-s.T_solidus)+s.cp_gliding;
        else
          cp :=s.cp_gliding;
        end if;
      end specificHeat;
    end Silicon;

    model Radiation
      "View factors are given as a continuous input.  Also inclused a possibility to block tunnel entrances"

    protected
      parameter Real sigma = Modelica.Constants.sigma;

    public
      Modelica.SIunits.HeatFlux J1, J2, J3;
      Modelica.SIunits.HeatFlowRate q12,q13,q23;

      parameter Modelica.SIunits.Area A1;
      parameter Modelica.SIunits.Area A2;
      parameter Modelica.SIunits.Area A3;
      parameter Real e1(min=0,max=1), e2(min=0,max=1), e3(min=0,max=1);

      Real F12new(min=0,max=1), F13new(min=0,max=1), F23new(min=0,max=1);

      Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a port_1
        annotation (Placement(transformation(extent={{-10,-70},{10,-50}}),
            iconTransformation(extent={{-10,-70},{10,-50}})));
      Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a port_2
        annotation (Placement(transformation(extent={{80,40},{100,60}}),
            iconTransformation(extent={{80,40},{100,60}})));
      Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a port_3
        annotation (Placement(transformation(extent={{-100,40},{-80,60}}),
            iconTransformation(extent={{-100,40},{-80,60}})));

      Modelica.Blocks.Interfaces.BooleanInput isBlocked annotation (Placement(
            transformation(
            extent={{-10,-10},{10,10}},
            rotation=270,
            origin={20,70}),iconTransformation(
            extent={{-10,-10},{10,10}},
            rotation=270,
            origin={20,70})));
      Modelica.Blocks.Interfaces.RealInput F12 annotation (Placement(transformation(
            extent={{-20,-20},{20,20}},
            rotation=270,
            origin={-70,80}), iconTransformation(
            extent={{-10,-10},{10,10}},
            rotation=270,
            origin={-60,70})));
      Modelica.Blocks.Interfaces.RealInput F13 annotation (Placement(transformation(
            extent={{-20,-20},{20,20}},
            rotation=270,
            origin={-30,80}), iconTransformation(
            extent={{-10,-10},{10,10}},
            rotation=270,
            origin={-20,70})));
      Modelica.Blocks.Interfaces.RealInput F23 annotation (Placement(transformation(
            extent={{-20,-20},{20,20}},
            rotation=270,
            origin={50,80}), iconTransformation(
            extent={{-10,-10},{10,10}},
            rotation=270,
            origin={60,70})));
    equation
      if isBlocked then //if tunnel openings are blocked, assuming some radiation will still leak.

        F12new = 0.95;
        F13new = 0.05;
        F23new = 0.05;
      else
        F12new = F12;
        F13new = F13;
        F23new = F23;
      end if;

      (sigma*port_1.T^4-J1)/((1-e1)/(e1*A1)) = (J1-J2)/(1/(A1*F12new)) + (J1-J3)/(1/(A1*F13new));
      (sigma*port_2.T^4-J2)/((1-e2)/(e2*A2)) = (J2-J1)/(1/(A1*F12new)) + (J2-J3)/(1/(A2*F23new));
      (sigma*port_3.T^4-J3)/((1-e3)/(e3*A3)) = (J3-J1)/(1/(A1*F13new)) + (J3-J2)/(1/(A2*F23new));

      q12 = (J1-J2)/(1/(A1*F12new));
      q13 = (J1-J3)/(1/(A1*F13new));
      q23 = (J2-J3)/(1/(A2*F23new));

      port_1.Q_flow = (sigma*port_1.T^4-J1)/((1-e1)/(e1*A1));
      port_2.Q_flow = (sigma*port_2.T^4-J2)/((1-e2)/(e2*A2));
      port_3.Q_flow = (sigma*port_3.T^4-J3)/((1-e3)/(e3*A3));

      annotation (Icon(coordinateSystem(preserveAspectRatio=false), graphics={
            Text(
              extent={{26,-40},{84,-54}},
              lineColor={238,46,47},
              textString="Hot Source"),
            Text(
              extent={{80,20},{144,0}},
              lineColor={28,108,200},
              textString="Heat Collector"),
            Text(
              extent={{-124,20},{-80,0}},
              lineColor={0,0,0},
              textString="Ambient"),
            Line(points={{-90,40},{-58,0},{-60,-4},{-50,0},{-58,-10},{-46,-6},{
                  -52,-16},{-42,-12},{-48,-22},{-38,-18},{-40,-22},{-10,-60}},
                color={0,0,0}),
            Line(points={{10,-60},{40,-22},{40,-18},{46,-22},{42,-12},{50,-18},
                  {46,-8},{54,-12},{50,-4},{58,-8},{54,0},{58,0},{90,40}},
                color={0,0,0}),
            Line(points={{80,50},{16,50},{14,54},{12,46},{8,54},{4,46},{0,54},{
                  -4,46},{-8,54},{-12,46},{-16,54},{-20,46},{-22,50}}, color={0,
                  0,0}),
            Line(points={{-22,50},{-80,50}}, color={0,0,0}),
            Text(
              extent={{-58,40},{60,0}},
              lineColor={0,0,0},
              textString="%name")}),
                           Diagram(coordinateSystem(preserveAspectRatio=false)));
    end Radiation;

    model TunnelSegment
      "Tunnel view factors depend on time and relative positioning, but with the possibility of blocking of tunnel entry and exit to ensure all radiation impacts the tunnel wall. Alo, optional wall-loss"

    protected
      parameter Modelica.SIunits.Mass m_air = length*A3/2*air.rho "Total air in entire tunnel segment";
      parameter Modelica.SIunits.Mass m_HTF = length*Modelica.Constants.pi*d^2/4*HTF.rho "Total HTF in entire tunnel segment";

      //antar helt like view-factors i hvert segment
      parameter Real F12 = Components.ViewFactorParallelRectangles(width, length, height) +
        2*Components.ViewFactorPerpendicularRectangles(width, height, length);
      parameter Real F13 = 2*Components.ViewFactorPerpendicularRectangles(length, height, width);
      parameter Real F32 = 1 - Components.ViewFactorParallelRectangles(height, width, length) -
        Components.ViewFactorPerpendicularRectangles(height, length, width);
      parameter Real F23 = F32*A3/A2; // fra collector ut begge sidene av skjoldet

    public
      parameter Modelica.Thermal.FluidHeatFlow.Media.Medium HTF=Modelica.Thermal.FluidHeatFlow.Media.Medium() "Heat Transfer FLuid"
        annotation(choicesAllMatching=true);
      parameter Modelica.Thermal.FluidHeatFlow.Media.Medium air=Modelica.Thermal.FluidHeatFlow.Media.Medium() "air used in forced convection inside tunnel"
        annotation(choicesAllMaching=true);

      final parameter Modelica.SIunits.Area      A1 = length*width;
      final parameter Modelica.SIunits.Area      A2 = length*width + 2*length*height;
      final parameter Modelica.SIunits.Area      A3 = 2*width * height;  //A3 tilsvarer et hull på en side av segmentet
      parameter Real                       e1, e2, e3;

      parameter Integer                      N(min=2) "Number of partitions in segment";
      parameter Modelica.SIunits.Temperature T0_HTF;
      parameter Modelica.SIunits.Temperature T0_air;
      parameter Modelica.SIunits.Length      length, width, height;

      parameter Real startTime "position of tunnelSegment in discrete tunnel, i_mould/N_mould";
      parameter Real active(min=0, max=100)=50;
      parameter Real period(min=0) = 7200;
      parameter Real pouring(min=0, max=period)=1800;

      parameter Modelica.SIunits.Diameter    d "diameter of HTF pipe";

      Modelica.SIunits.CoefficientOfHeatTransfer h; //,h_forced

      parameter Boolean useWallLoss=false "Enable / disable volume flow input"
        annotation(Evaluate=true);
      parameter Modelica.SIunits.CoefficientOfHeatTransfer h_free = 6;
      parameter Modelica.SIunits.ThermalConductivity k_ins = 0.09;
      parameter Modelica.SIunits.Length L_ins = 0.005;
      Boolean blocked;

      Modelica.SIunits.HeatFlowRate Q_mould, Q_airloss, Q_HTF, Q_radloss, Q_wallloss;
      Modelica.SIunits.Temperature Tair_a, Tair_b, Thtf_a, Thtf_b;

      Modelica.Thermal.FluidHeatFlow.Interfaces.FlowPort_a flowPort_a(medium=air)
        annotation (Placement(transformation(extent={{-110,-30},{-90,-10}}),
            iconTransformation(extent={{-110,-30},{-90,-10}})));
      Modelica.Thermal.FluidHeatFlow.Interfaces.FlowPort_b flowPort_b(medium=air)
        annotation (Placement(transformation(extent={{88,-30},{108,-10}}),
            iconTransformation(extent={{88,-30},{108,-10}})));
      Modelica.Thermal.FluidHeatFlow.Interfaces.FlowPort_a flowPort_a1(medium=HTF)
        annotation (Placement(transformation(extent={{-110,30},{-90,50}}),
            iconTransformation(extent={{-110,30},{-90,50}})));
      Modelica.Thermal.FluidHeatFlow.Interfaces.FlowPort_b flowPort_b1(medium=HTF)
        annotation (Placement(transformation(extent={{90,30},{110,50}}),
            iconTransformation(extent={{90,30},{110,50}})));
      Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a port_mould
        annotation (Placement(transformation(extent={{-10,-80},{10,-60}}),
            iconTransformation(extent={{-10,-80},{10,-60}})));
      Radiation radiation[N](
        each A1=A1/N,
        each A2=A2/N,
        each A3=A3/N,
        each e1=e1,
        each e2=e2,
        each e3=e3)
        annotation (Placement(transformation(extent={{20,-26},{78,34}})));
      Modelica.Thermal.HeatTransfer.Components.Convection convection[N] annotation (
          Placement(transformation(
            extent={{-10,-10},{10,10}},
            rotation=90,
            origin={-40,-58})));
      Modelica.Thermal.HeatTransfer.Components.Convection convection1[N] annotation (
          Placement(transformation(
            extent={{10,-10},{-10,10}},
            rotation=90,
            origin={-40,10})));
      Modelica.Thermal.FluidHeatFlow.Components.HeatedPipe HTFflow[N](
        each medium=HTF,
        each V_flowLaminar=1,
        each V_flowNominal=2,
        each h_g=0,
        each T0fixed=true,
        each T0=T0_HTF,
        each m=m_HTF/N,
        each dpLaminar=100,
        each dpNominal=2000)
        annotation (Placement(transformation(extent={{-32,30},{-12,50}})));
      Modelica.Thermal.FluidHeatFlow.Components.HeatedPipe airFlow[N](
        each medium=air,
        each V_flowLaminar=1,
        each V_flowNominal=2,
        each h_g=0,
        each T0fixed=true,
        each T0=T0_air,
        each m=m_air/N,
        each dpLaminar=100,
        each dpNominal=2000)
        annotation (Placement(transformation(extent={{-70,-30},{-50,-10}})));
      Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_b port_ambient
        annotation (Placement(transformation(extent={{70,90},{90,110}}),
            iconTransformation(extent={{70,90},{90,110}})));
      Modelica.Blocks.Interfaces.RealInput convectionCoefficient(unit="W/(m2.K)") annotation (
          Placement(transformation(
            extent={{-20,-20},{20,20}},
            rotation=270,
            origin={-84,100}),iconTransformation(
            extent={{-10,-10},{10,10}},
            rotation=270,
            origin={-80,78})));
      Modelica.Blocks.Math.Gain Gc1(k(unit="m2")=A2/N, y(unit="W/K"))
        annotation (Placement(transformation(extent={{-76,0},{-56,20}})));
      Modelica.Blocks.Math.Gain Gc(k(unit="m2")=A1/N, y(unit="W/K"))
        annotation (Placement(transformation(extent={{-78,-68},{-58,-48}})));
      Modelica.Thermal.HeatTransfer.Components.ThermalResistor [N] wallLoss(
        each R=1/(h_free*A2/N)+1/(k_ins*A2/N/L_ins)) if useWallLoss  annotation (Placement(transformation(extent={{38,50},
                {58,70}})));
      ViewFactor viewFactorF12(
        pouring=pouring,
        active=active,
        period=period,
        startTime=startTime,
        offset=F12,
        amplitude=1 - F12 - 0.01)
        annotation (Placement(transformation(extent={{-56,80},{-36,100}})));
      ViewFactor viewFactorF13(
        pouring=pouring,
        active=active,
        period=period,
        startTime=startTime,
        offset=F13,
        amplitude=-F13 + 0.01)
        annotation (Placement(transformation(extent={{-28,80},{-8,100}})));
      ViewFactor viewFactorF23(
        pouring=pouring,
        active=active,
        period=period,
        startTime=startTime,
        offset=F23,
        amplitude=-F23 + 0.01)
        annotation (Placement(transformation(extent={{0,80},{20,100}})));
    equation

      blocked = if abs(flowPort_a.m_flow)>0.1*height*width then false else true;

      h=convectionCoefficient;

      Q_mould = port_mould.Q_flow;
      Q_airloss = sum(airFlow[1:N].heatPort.Q_flow);
      Q_HTF = sum(HTFflow[1:N].heatPort.Q_flow);
      Q_radloss = -sum(radiation[1:N].port_3.Q_flow); //-port_ambient.Q_flow;
      //Q_wallloss = if useWallLoss then sum(wallLoss[1:N].port_a.Q_flow) else 0;
      Q_wallloss = -port_ambient.Q_flow-Q_radloss;
      Tair_a = airFlow[1].T_a;
      Tair_b = airFlow[end].T_b;
      Thtf_a = HTFflow[1].T_a;
      Thtf_b = HTFflow[end].T_b;

      connect(convectionCoefficient, Gc1.u)
        annotation (Line(points={{-84,100},{-84,10},{-78,10}},color={0,0,127}));
      connect(convectionCoefficient, Gc.u) annotation (Line(points={{-84,100},{-84,-58},
              {-80,-58}},           color={0,0,127}));

      connect(flowPort_a, airFlow[1].flowPort_a);
      connect(flowPort_b, airFlow[end].flowPort_b);
      connect(flowPort_a1, HTFflow[1].flowPort_a);
      connect(flowPort_b1, HTFflow[end].flowPort_b);

      for i in 1:N-1 loop
        connect(airFlow[i].flowPort_b, airFlow[i+1].flowPort_a);
        connect(HTFflow[i].flowPort_b, HTFflow[i+1].flowPort_a);
      end for;

      for i in 1:N loop

        //convection connections first
        connect(Gc.y,convection[i].Gc);
        connect(Gc1.y, convection1[i].Gc);
        connect(port_mould, convection[i].solid);
        connect(convection[i].fluid, airFlow[i].heatPort);
        connect(airFlow[i].heatPort, convection1[i].fluid);
        connect(convection1[i].solid, HTFflow[i].heatPort);

        //radiation connections
        connect(port_mould, radiation[i].port_1);
        connect(radiation[i].port_2, HTFflow[i].heatPort);
        connect(radiation[i].port_3, port_ambient);
        radiation[i].isBlocked=blocked;
        connect(viewFactorF12.y, radiation[i].F12);
        connect(viewFactorF13.y, radiation[i].F13);
        connect(viewFactorF23.y, radiation[i].F23);

        //tunnel losses
        connect(wallLoss[i].port_a, HTFflow[i].heatPort);
        connect(wallLoss[i].port_b, port_ambient);

      end for;

                                                                       annotation(Dialog(enable= useWallLoss),
                   Icon(coordinateSystem(preserveAspectRatio=false), graphics={
            Rectangle(
              extent={{-100,68},{100,-52}},
              lineColor={238,46,47},
              fillColor={215,215,215},
              fillPattern=FillPattern.HorizontalCylinder),
            Rectangle(
              extent={{-100,-52},{100,-60}},
              lineColor={0,0,0},
              fillColor={238,46,47},
              fillPattern=FillPattern.CrossDiag),
            Rectangle(
              extent={{-100,20},{100,10}},
              lineColor={0,0,0},
              fillColor={135,135,135},
              fillPattern=FillPattern.Solid),
            Rectangle(
              extent={{-100,68},{100,58}},
              lineColor={0,0,0},
              fillColor={135,135,135},
              fillPattern=FillPattern.Solid),
            Rectangle(
              extent={{-100,58},{100,20}},
              lineColor={28,108,200},
              fillColor={170,213,255},
              fillPattern=FillPattern.Solid),
            Line(points={{0,31},{0,-31},{-18,-11},{0,-31},{18,-11}},   color={238,46,
                  47},
              origin={-72,-21},
              rotation=180),
            Line(points={{0,31},{0,-31},{-18,-11},{0,-31},{18,-11}},   color={238,46,
                  47},
              origin={0,-21},
              rotation=180),
            Line(points={{0,31},{0,-31},{-18,-11},{0,-31},{18,-11}},   color={238,46,
                  47},
              origin={70,-21},
              rotation=180),
            Line(points={{0,31},{0,-31},{-18,-11},{0,-31},{18,-11}},   color={28,108,
                  200},
              origin={-10,39},
              rotation=90),
            Line(points={{-176,44}}, color={28,108,200}),
            Line(points={{-60,68},{-60,-60}}, color={255,255,255}),
            Line(points={{-20,68},{-20,-60}}, color={255,255,255}),
            Line(points={{20,68},{20,-60}}, color={255,255,255}),
            Line(points={{60,68},{60,-60}}, color={255,255,255}),
            Text(
              extent={{-106,90},{-86,66}},
              lineColor={28,108,200},
              textString="h"),
            Text(
              extent={{-62,100},{58,68}},
              lineColor={0,0,0},
              textString="%name")}),                                 Diagram(
            coordinateSystem(preserveAspectRatio=false), graphics={
            Line(points={{0,-70},{-40,-70},{-40,-68},{-40,-48},{-40,-30},{-60,-30},{
                  -40,-30},{-40,0},{-40,-6}}, color={238,46,47}),
            Line(points={{-40,20},{-40,28},{-40,30},{0,30}}, color={238,46,47}),
            Line(points={{2,-70},{38,-70},{48,-70},{48,-68},{48,-12},{50,-14},{48,-14}},
                color={238,46,47}),
            Line(points={{76,18},{76,30},{0,30}}, color={238,46,47}),
            Line(points={{22,16},{22,86},{80,86},{80,96},{80,96}}, color={238,46,47}),
            Line(points={{-98,40},{-10,40},{98,40}}, color={0,0,0}),
            Line(points={{-98,-20},{98,-20}}, color={0,0,0}),
            Line(points={{-8,30},{-8,60},{-4,60},{80,60},{80,100}}, color={238,46,47})}));
    end TunnelSegment;

    function ViewFactorParallelRectangles
      "l and w is the size of each rectangle, d is the distance between them, as 
  described on http://www.thermalradiation.net/calc/sectionc/C-11.html"

      input Modelica.SIunits.Length a;
      input Modelica.SIunits.Length b;
      input Modelica.SIunits.Distance c;
      output Real F;

    protected
      Real X,Y,p1,p2,p3,p4;
    algorithm
      X :=a/c;
      Y :=b/c;
      p1 :=log(sqrt(((1 + X^2)*(1 + Y^2))/(1 + X^2 + Y^2)));
      p2 :=X*sqrt(1 + Y^2)*atan(X/sqrt(1 + Y^2));
      p3 :=Y*sqrt(1 + X^2)*atan(Y/sqrt(1 + X^2));
      p4 :=-X*atan(X) - Y*atan(Y);
      F :=2/(Modelica.Constants.pi*X*Y)*(p1+p2+p3+p4);

    end ViewFactorParallelRectangles;

    function ViewFactorPerpendicularRectangles "l and w is th esize of each rectangle, d is the distance between them, as 
  described on http://www.thermalradiation.net/calc/sectionc/C-11.html"

      input Modelica.SIunits.Length w;
      input Modelica.SIunits.Length h;
      input Modelica.SIunits.Distance l;
      output Real F;

    protected
      Real H,W,p1,p2,p3,p4;
    algorithm
      H :=h/l;
      W :=w/l;
      p1 :=W*atan(1/W) + H*atan(1/H) - (H^2 + W^2)^0.5*atan(1/sqrt(H^2 + W^2));
      p2 :=((1 + W^2)*(1 + H^2))/(1 + W^2 + H^2);
      p3 :=((W^2*(1 + W^2 + H^2))/((1 + W^2)*(W^2 + H^2)))^(W^2);
      p4 :=((H^2*(1 + H^2 + W^2))/((1 + H^2)*(H^2 + W^2)))^(H^2);
      F :=1/(Modelica.Constants.pi*W)*(p1 + 1/4*log(p2*p3*p4));
    end ViewFactorPerpendicularRectangles;

    model TunnelDiscrete
      "Same as TunnelDiscrete2, but with the possibility of blocking of tunnel entry and exit to ensure all radiation impacts the tunnel wall"

      parameter Modelica.Thermal.FluidHeatFlow.Media.Medium air=Modelica.Thermal.FluidHeatFlow.Media.Air_70degC()
        annotation(choicesAllMatching=true);
      parameter Modelica.Thermal.FluidHeatFlow.Media.Medium HTF=Modelica.Thermal.FluidHeatFlow.Media.Medium()
        annotation(choicesAllMatching=true);
      parameter Silicon.SiliconParameters s=Silicon.SiliconParameters();

      parameter Integer N_moulds(min=2)=2 "Number of discrete moulds in the tunnel";
      parameter Integer N_partitions(min=1)=1 "Number of tunnel partitions per mould";
      parameter Integer N_mouldlayers(min=1)=1 "Number of layers in moulds";

      parameter Real active(min=0, max=100)=50 "The fraction of the period in which the mould is heating the cycle, in percentage, 0-100";
      parameter Real period(min=0) = 7200 "Time in seconds for one melting cylce";
      parameter Real pouring(min=0, max=period)=1800 "Time to pour one ladle into 9 moulds";

      parameter Real e1(min=0, max=1) = 0.4;
      parameter Real e2(min=0, max=1) = 0.5;
      parameter Real e3(min=0, max=1) = 0.9999;

      parameter Modelica.SIunits.Temperature  T_amb "ambient temperature=air inlet temperature";
      parameter Modelica.SIunits.Temperature  T0_mould "mould initial temperature";
      parameter Modelica.SIunits.Temperature  T0_HTF "HTF intial temperature";

      parameter Modelica.SIunits.Mass m_silicon = 9000 "Total mass of silicon";
      parameter Modelica.SIunits.Length length = N_moulds*2.03 "Total length of tunnel";
      parameter Modelica.SIunits.Length width = 1.8 "width of tunnel";
      parameter Modelica.SIunits.Length height = 1 "height of tunnel, mould surface to roof";
      parameter Modelica.SIunits.Diameter d = 0.1 "diameter of HTF pipe";
      parameter Modelica.SIunits.Velocity v_air = 1 "air-velocity through tunnel" annotation(Dialog(enable=not useVolumeFlowInput));
      parameter Modelica.SIunits.CoefficientOfHeatTransfer h_free = 6 "air free convection coefficient";
      parameter Modelica.SIunits.ThermalConductivity k_ins = 0.09;
      parameter Modelica.SIunits.Length L_ins = 0.01;
      parameter Boolean useVolumeFlowInput=false "Enable / disable volume flow input" annotation(Evaluate=true);
      parameter Boolean useWallLoss=false "Enable / disable tunnel wall heat loss" annotation(Evaluate=true);
      parameter Boolean measureSolidification=false "Measure the innermost temperature of last silicon mould" annotation(Evaluate=true);

      Modelica.SIunits.Temperature Tair_a, Tair_b, Thtf_a, Thtf_b;
      Modelica.SIunits.HeatFlowRate Q_mould, Q_HTF, Q_airloss, Q_radloss, Q_wallloss;
      Modelica.SIunits.Heat E_mould(start=0), E_HTF(start=0), E_airloss(start=0), E_radloss(start=0), E_wallloss(start=0);

      TunnelSegment tunnelSegment[N_moulds](
        each HTF=HTF,
        each air=air,
        each N=N_partitions,
        each T0_HTF=T0_HTF,
        each T0_air=T_amb,
        each width=width,
        each height=height,
        each e1=e1,
        each e2=e2,
        each e3=e3,
        each d=d,
        each length=length/N_moulds,
        each h_free=h_free,
        each useWallLoss=useWallLoss,
        startTime=linspace(
                1,
                pouring,
                N_moulds),
        each active=active,
        each period=period,
        each pouring=pouring,
        each k_ins=k_ins,
        each L_ins=L_ins)
        annotation (Placement(transformation(extent={{-16,-16},{24,24}})));
      Silicon.Mould mould[N_moulds](
        each m=m_silicon/N_moulds,
        each N=N_mouldlayers,
        each A=length/N_moulds*width,
        each s=s,
        each T0=T0_mould)
        annotation (Placement(transformation(extent={{-18,-78},{2,-58}})));
      Modelica.Blocks.Sources.Constant volumeFlow(k=v_air*height*width) if not
        useVolumeFlowInput
        annotation (Placement(transformation(extent={{-112,2},{-92,22}})));
      Modelica.Thermal.FluidHeatFlow.Interfaces.FlowPort_a flowPort_a(medium=HTF)
        annotation (Placement(transformation(extent={{-110,30},{-90,50}}),
            iconTransformation(extent={{-110,30},{-90,50}})));
      Modelica.Thermal.FluidHeatFlow.Interfaces.FlowPort_b flowPort_b(medium=HTF)
        annotation (Placement(transformation(extent={{90,30},{110,50}}),
            iconTransformation(extent={{90,30},{110,50}})));
      ForcedConvection forcedConvection(
         medium=air,
         width=width,
         height=height,
         h_free=h_free)
         annotation (Placement(transformation(extent={{-44,10},{-24,-10}})));
      Modelica.Thermal.FluidHeatFlow.Sources.Ambient airInlet(
        constantAmbientPressure=0,
        medium=air,
        constantAmbientTemperature=T_amb)
        annotation (Placement(transformation(extent={{-82,-30},{-102,-10}})));
      Modelica.Thermal.FluidHeatFlow.Sources.Ambient airOutlet(
        medium=air,
        constantAmbientPressure=0,
        constantAmbientTemperature=T_amb)
        annotation (Placement(transformation(extent={{70,-30},{90,-10}})));
      Modelica.Thermal.FluidHeatFlow.Sources.VolumeFlow pump1(
        m=0,
        useVolumeFlowInput=true,
        constantVolumeFlow=1,
        medium=air)
        annotation (Placement(transformation(extent={{-72,-30},{-52,-10}})));
      Modelica.Thermal.HeatTransfer.Sources.FixedTemperature ambient(T=T_amb)
        annotation (Placement(transformation(
            extent={{-10,-10},{10,10}},
            rotation=270,
            origin={0,70})));
      Modelica.Blocks.Sources.BooleanPulse booleanMultiple[N_moulds](
        each width=active,
        each period=period,
        startTime=linspace(
            1,
            pouring,
            N_moulds)) if     N_moulds >= 2
        annotation (Placement(transformation(extent={{-68,-76},{-48,-56}})));
      HeatBlocker heatBlocker[N_moulds]
        annotation (Placement(transformation(extent={{-6,-56},{14,-36}})));
      Modelica.Blocks.Interfaces.RealInput airVolumeFlow if useVolumeFlowInput
        annotation (Placement(transformation(extent={{-118,64},{-78,104}}),
            iconTransformation(
            extent={{-10,-10},{10,10}},
            rotation=0,
            origin={-110,-20})));
      Modelica.Blocks.Interfaces.RealOutput lastMouldTemp annotation (Placement(
          transformation(
            extent={{-10,-10},{10,10}},
            rotation=270,
            origin={-90,-70}),
          iconTransformation(
            extent={{-10,-10},{10,10}},
            rotation=270,
            origin={-82,-70}),
          visible=DynamicSelect(true, measureSolidification)));
      Modelica.Blocks.Interfaces.RealOutput firstMouldTemp
                                                          annotation (Placement(
          transformation(
            extent={{-10,-10},{10,10}},
            rotation=270,
            origin={-114,-70}),
          iconTransformation(
            extent={{-10,-10},{10,10}},
            rotation=270,
            origin={84,-70}),
          visible=DynamicSelect(true, measureSolidification)));
    equation

      //summary:
      Tair_a = tunnelSegment[1].airFlow[1].T_a;
      Tair_b = tunnelSegment[end].airFlow[end].T;
      Thtf_a = tunnelSegment[1].HTFflow[1].T_a;
      Thtf_b = tunnelSegment[end].HTFflow[end].T;

      Q_mould = sum(tunnelSegment[1:N_moulds].Q_mould);
      Q_HTF = sum(tunnelSegment[1:N_moulds].Q_HTF);
      Q_airloss = sum(tunnelSegment[1:N_moulds].Q_airloss);
      Q_radloss = sum(tunnelSegment[1:N_moulds].Q_radloss);
      Q_wallloss = sum(tunnelSegment[1:N_moulds].Q_wallloss);

      der(E_mould) = Q_mould;
      der(E_HTF) = Q_HTF;
      der(E_airloss) = Q_airloss;
      der(E_radloss) = Q_radloss;
      der(E_wallloss) = Q_wallloss;

      lastMouldTemp = mould[end].heatCapSilicon[1].T;
      firstMouldTemp = mould[1].heatCapSilicon[1].T;

      //connections
      for i in 1:N_moulds-1 loop
        connect(tunnelSegment[i].flowPort_b, tunnelSegment[i+1].flowPort_a);
        connect(tunnelSegment[i].flowPort_b1, tunnelSegment[i+1].flowPort_a1);
      end for;

      for i in 1:N_moulds loop
        // connect(mould[i].port_a, tunnelSegment[i].port_mould);
        connect(mould[i].port_a, heatBlocker[i].port_a);
        connect(heatBlocker[i].port_b, tunnelSegment[i].port_mould);

        connect(tunnelSegment[i].port_ambient, ambient.port);
        connect(forcedConvection.y, tunnelSegment[i].convectionCoefficient);

        connect(booleanMultiple[i].y, mould[i].reStart);
        connect(booleanMultiple[i].y, heatBlocker[i].u);

      end for;

      connect(flowPort_a, tunnelSegment[1].flowPort_a1) annotation (Line(points={{-100,40},
              {-48,40},{-48,12},{-16,12}},     color={255,0,0}));
      connect(tunnelSegment[end].flowPort_b1, flowPort_b) annotation (Line(points={{24,12},
              {40,12},{40,40},{100,40}},     color={255,0,0}));
      connect(tunnelSegment[end].flowPort_b, airOutlet.flowPort)
        annotation (Line(points={{23.6,0},{46,0},{46,-20},{70,-20}},
                                                       color={255,0,0}));
      connect(pump1.flowPort_b, forcedConvection.flowPort_a)
        annotation (Line(points={{-52,-20},{-50,-20},{-50,0},{-44,0}},
                                                   color={255,0,0}));
      connect(forcedConvection.flowPort_b, tunnelSegment[1].flowPort_a)
        annotation (Line(points={{-24,0},{-16,0}}, color={255,0,0}));
      connect(volumeFlow.y, pump1.volumeFlow)
        annotation (Line(points={{-91,12},{-62,12},{-62,-10}},   color={0,0,127}));
      connect(airInlet.flowPort, pump1.flowPort_a)
        annotation (Line(points={{-82,-20},{-72,-20}}, color={255,0,0}));
      connect(airVolumeFlow, pump1.volumeFlow)
        annotation (Line(points={{-98,84},{-62,84},{-62,-10}}, color={0,0,127}));
     annotation (Icon(coordinateSystem(preserveAspectRatio=false, extent={{-120,-100},
                {120,100}}),                                       graphics={
              Rectangle(extent={{-100,60},{100,-40}}, lineColor={238,46,47},
              fillColor={255,255,255},
              fillPattern=FillPattern.HorizontalCylinder),
              Rectangle(extent={{-100,60},{100,20}},  lineColor={28,108,200},
              fillColor={255,255,255},
              fillPattern=FillPattern.HorizontalCylinder),
            Rectangle(
              extent={{-100,18},{100,22}},
              lineColor={0,0,0},
              lineThickness=0.5,
              fillColor={0,0,0},
              fillPattern=FillPattern.Solid),
            Rectangle(
              extent={{-100,-40},{-50,-60}},
              lineColor={0,0,0},
              fillPattern=FillPattern.CrossDiag,
              fillColor={238,46,47}),
            Line(points={{-4.65367e-015,48},{0,-10},{-18,8},{0,-10},{18,8}},
                                                                           color={238,
                  46,47},
              origin={-80,8},
              rotation=180,
              thickness=0.5),
            Line(points={{-3.67398e-015,30},{0,-10},{-10,-1.68234e-016},{0,-10},{10,
                  -2.75855e-016}},                                         color={238,
                  46,47},
              origin={-52,40},
              rotation=90,
              thickness=0.5),
            Line(points={{-4.8986e-015,50},{0,-10},{-4,-6},{0,-10},{4,-6}},color={28,
                  108,200},
              origin={-24,-12},
              rotation=90,
              thickness=0.5),
            Line(points={{-4.8986e-015,50},{0,-10},{-4,-6},{0,-10},{4,-6}},color={28,
                  108,200},
              origin={64,-12},
              rotation=90,
              thickness=0.5),
            Line(points={{-4.65367e-015,48},{0,-10},{-18,8},{0,-10},{18,8}},
                                                                           color={238,
                  46,47},
              origin={0,8},
              rotation=180,
              thickness=0.5),
            Line(points={{-4.65367e-015,48},{0,-10},{-18,8},{0,-10},{18,8}},
                                                                           color={238,
                  46,47},
              origin={76,8},
              rotation=180,
              thickness=0.5),
            Rectangle(
              extent={{-100,60},{100,64}},
              lineColor={0,0,0},
              lineThickness=0.5,
              fillColor={0,0,0},
              fillPattern=FillPattern.Solid),
            Rectangle(
              extent={{-26,-40},{26,-60}},
              lineColor={0,0,0},
              fillPattern=FillPattern.CrossDiag,
              fillColor={238,46,47}),
            Rectangle(
              extent={{52,-40},{100,-60}},
              lineColor={0,0,0},
              fillPattern=FillPattern.CrossDiag,
              fillColor={238,46,47}),
            Text(
              extent={{-100,100},{100,64}},
              lineColor={0,0,0},
              fillColor={238,46,47},
              fillPattern=FillPattern.CrossDiag,
              textString="%name"),
            Text(
              extent={{-26,54},{28,28}},
              lineColor={0,0,0},
              fillColor={238,46,47},
              fillPattern=FillPattern.CrossDiag,
              textString="HTF flow"),
            Text(
              extent={{-36,-16},{40,-38}},
              lineColor={0,0,0},
              fillColor={238,46,47},
              fillPattern=FillPattern.CrossDiag,
              textString="Air-flow"),
            Line(points={{-3.67398e-015,30},{0,-10},{-10,-1.68234e-016},{0,-10},{10,
                  -2.75855e-016}},                                         color={238,
                  46,47},
              origin={70,40},
              rotation=90,
              thickness=0.5)}),                                     Diagram(
            coordinateSystem(preserveAspectRatio=false, extent={{-120,-100},{120,100}}),
                                                         graphics={Line(points={{-34,
                  10},{-34,26},{-34,26},{-20,26},{-12,26},{-12,20}},    color={28,108,
                  200}),
            Line(points={{-8,-78},{-8,-86},{4,-86},{4,-52},{4,-44},{4,-10}}, color={
                  238,46,47}),
            Line(points={{20,24},{20,30},{0,30},{0,60}},color={238,46,47})}));
    end TunnelDiscrete;

    model ForcedConvection
      "Gnielsky-correlation for forced convection coefficient"

      extends Modelica.Thermal.FluidHeatFlow.Interfaces.Partials.FlowSensor(y(unit="W/(m2.K)")
          "Volume flow as output signal");

      parameter Modelica.SIunits.Length width, height;
      parameter Modelica.SIunits.CoefficientOfHeatTransfer h_free;

      Real Re, Pr, Nu, f;
      Modelica.SIunits.Diameter d_hyd;
      Modelica.SIunits.CoefficientOfHeatTransfer h_final, h_forced;
      Modelica.SIunits.Velocity v_air;

    equation
      d_hyd = 4*(width*height)/(2*(width+height)); //=4*A/P
      v_air = V_flow/(width*height);
      Re = abs(v_air*d_hyd/medium.nue);
      Pr = medium.cp*medium.rho*medium.nue/medium.lamda;
      if Re <= 2300 then
        f=0;
        Nu=0;
        h_forced=0;
      else
        f = (0.79*log(Re)-1.64)^(-2); //Darcy friction factor, smoot tube friction factor by petukhov
        Nu = ((f/8)*(Re-1000)*Pr)/(1+12.7*(f/8)^0.5*(Pr^(2/3)-1));
        h_forced = Nu*medium.lamda/d_hyd;
      end if;

      h_final = max(h_forced, h_free);

      y = h_final;
    annotation (Documentation(info="<html>
V_flowSensor measures the volume flow rate.<br>
Thermodynamic equations are defined by Partials.FlowSensor.
</html>"),
      Icon(coordinateSystem(preserveAspectRatio=true, extent={{-100,-100},{100,100}}),
        graphics={Text(extent={{-22,-20},{20,-60}},
              lineColor={0,0,255},
              textString="h")}));
    end ForcedConvection;

    model StorageHeatTransfer "heat transfer according to Yian"

      extends Modelica.Thermal.FluidHeatFlow.Interfaces.Partials.FlowSensor(y(unit="W/(m2.K)")
          "Volume flow as output signal");

      parameter Modelica.SIunits.Diameter di;
      parameter Modelica.SIunits.Diameter do;
      parameter Modelica.SIunits.ThermalConductivity k;
      parameter Real N_pipes;

      Real Re, Pr, Nu, f, Bi;
      Modelica.SIunits.CoefficientOfHeatTransfer h_forced, h_E;// h_geo;
      Modelica.SIunits.Velocity v;
      Modelica.SIunits.Diameter a,b;

    equation
      //calculating forced convection heat transfer coefficient
      v = V_flow/(Modelica.Constants.pi*di^2/4*N_pipes);
      Re = abs(v*di/medium.nue);
      Pr = medium.cp*medium.rho*medium.nue/medium.lamda;
      if Re <= 2300 then
        f = 0;
        Nu = 4.36 "Uniform surface heat flux"; //=3.66 "uniform surface temperature";
        h_forced = Nu*medium.lamda/di;
      else
        f = (0.79*log(Re)-1.64)^(-2); //Darcy friction factor, smoot tube friction factor by petukhov
        Nu = ((f/8)*(Re-1000)*Pr)/(1+12.7*(f/8)^0.5*(Pr^(2/3)-1));
        h_forced = Nu*medium.lamda/di;
      end if;

      //calculating TES heat transfer coefficient
      a = di/2;
      b = do/2;

      1/h_E = 1/h_forced + 1/k*(4*a*b^4*log(b/a)-3*a*b^4+4*a^3*b^2-a^5)/(4*(b^2-a^2)^2);
      y = h_E;

      Bi = h_E/medium.lamda*(b^2-a^2)/(2*a);
    annotation (Documentation(info="<html>
V_flowSensor measures the volume flow rate.<br>
Thermodynamic equations are defined by Partials.FlowSensor.
</html>"),
      Icon(coordinateSystem(preserveAspectRatio=true, extent={{-100,-100},{100,100}}),
        graphics={Text(extent={{-22,-20},{20,-60}},
              lineColor={0,0,255},
              textString="h")}));
    end StorageHeatTransfer;

    model HeatBlocker "heat can only pass through if boolean=true"
      Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a port_a
        annotation (Placement(transformation(extent={{-10,-50},{10,-30}})));
      Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_b port_b
        annotation (Placement(transformation(extent={{-10,30},{10,50}})));
      Modelica.Blocks.Interfaces.BooleanInput u
        annotation (Placement(transformation(extent={{-100,-20},{-60,20}}),
            iconTransformation(extent={{-100,-20},{-60,20}})));

      Modelica.SIunits.HeatFlowRate Q_flow;
      Modelica.SIunits.TemperatureDifference dT;

    equation
      dT = port_a.T - port_b.T;
      port_a.Q_flow = Q_flow;
      port_b.Q_flow = -Q_flow;

      if u then
        dT=0;
      else
        Q_flow=0;
      end if;

      annotation (Icon(coordinateSystem(preserveAspectRatio=false), graphics={
              Rectangle(
              extent={{-10,-30},{10,30}},
              lineColor={238,46,47},
              fillColor={238,46,47},
              fillPattern=FillPattern.Solid), Rectangle(
              extent={{80,-10},{-80,10}},
              lineColor={0,0,0},
              fillColor={0,0,0},
              fillPattern=FillPattern.Solid)}),                      Diagram(
            coordinateSystem(preserveAspectRatio=false)));
    end HeatBlocker;

    model TES
      "not modeling the steel pipe itself, as the heat transfer through the concrete will be the limiting factor of this heat transfer"

    public
      parameter Modelica.Thermal.FluidHeatFlow.Media.Medium medium=Modelica.Thermal.FluidHeatFlow.Media.Medium()
        "Medium in the component" annotation(choicesAllMatching=true);

      parameter Modelica.SIunits.Temperature  T0_HTF "starting temperature of the pipe";
      parameter Modelica.SIunits.Temperature  T0_first "starting temperature of the first storage module";
      parameter Modelica.SIunits.Temperature  T0_last "starting temperature of the last storage module";

      parameter Integer  N(min=1) "Number of discretizations";
      parameter Integer  N_pipes(min=1) "Number of pipes in pipe bundle";

      parameter Modelica.SIunits.HeatCapacity    C_tot;
      final parameter Modelica.SIunits.Length    L =  (C_tot/cp)/(N_pipes*Modelica.Constants.pi*(do^2-di^2)/4*rho) "Length of storage unit";
      parameter Modelica.SIunits.Length          D(min=di) "storage unit utilizes triangular tube pitch, D meter between pipe center axises";
      parameter Modelica.SIunits.Diameter        di "inner pipediameter/inner diameter of the energy storage unit";
      final parameter Modelica.SIunits.Diameter  do = sqrt(2*sqrt(3)*D^2/Modelica.Constants.pi) "outer pipediameter in the equivalent cylindrical storage";

      //hentet disse verdiene fra yian
      parameter Modelica.SIunits.SpecificHeatCapacity  cp = 1180 "specific heat capacity of the storage material";
      parameter Modelica.SIunits.Density               rho = 3100 "specific density of storage material";
      parameter Modelica.SIunits.ThermalConductivity   k = 2.65 "thermal conductivity of storage material";

      final parameter Modelica.SIunits.Mass m_HTF = Modelica.Constants.pi*di^2/4*L*N_pipes*medium.rho;

      Modelica.SIunits.HeatFlowRate        Q_tot "Total heat flux flowing into the TES";
      Modelica.SIunits.Energy              E_TES "Total stored heat";
      Modelica.SIunits.Temperature         T_in, T_out, T_first, T_last;
      Modelica.SIunits.Volume              V_solid, V_tot;
      Modelica.SIunits.ThermalConductance  UA;
      Modelica.SIunits.Mass                m "Total mass of storage material";
      Modelica.SIunits.Area                A_tot "total cross-sectional area";
      Real                                 y "fraction of cross sectional area consisting of tubes";

      Modelica.Thermal.HeatTransfer.Components.HeatCapacitor heatCapacitor[N](each C=C_tot/N)
        annotation (Placement(transformation(extent={{18,60},{38,80}})));
      Modelica.Thermal.FluidHeatFlow.Components.HeatedPipe HTFflow[N](
        each medium=medium,
        each T0=T0_HTF,
        each V_flowLaminar=1,
        each V_flowNominal=2,
        each m=m_HTF/N,
        each h_g=0,
        each T0fixed=true,
        each dpLaminar=100,
        each dpNominal=2000) annotation (Placement(transformation(extent={{18,10},
                {38,-10}})));
      Modelica.Thermal.HeatTransfer.Components.Convection heatTransfer[N] annotation (Placement(transformation(
            extent={{-10,10},{10,-10}},
            rotation=270,
            origin={28,38})));
      StorageHeatTransfer storageHeatTransfer(
        di=di,
        do=do,
        k=k,
        medium=medium,
        N_pipes=N_pipes)                      annotation (Placement(transformation(extent={{-42,10},
                {-22,-10}})));
      Modelica.Blocks.Math.Gain gain(k(unit="m2")=Modelica.Constants.pi*di*N_pipes*L/N, y(unit="W/K"))
        annotation (Placement(transformation(extent={{-24,28},{-4,48}})));
      Modelica.Thermal.FluidHeatFlow.Interfaces.FlowPort_a flowPort_a(medium=medium)
        annotation (Placement(transformation(extent={{-110,-70},{-90,-50}}), iconTransformation(extent={{-110,
                -70},{-90,-50}})));
      Modelica.Thermal.FluidHeatFlow.Interfaces.FlowPort_b flowPort_b(medium=medium)
        annotation (Placement(transformation(extent={{90,-70},{110,-50}}), iconTransformation(extent={{90,-70},
                {110,-50}})));

    initial equation
      for i in 1:N loop
        heatCapacitor[i].T=T0_first+(i-1)*(T0_last-T0_first)/(N-1);
      end for;

    equation
      //equations
      Q_tot = sum(heatCapacitor[1:N].port.Q_flow);
      T_in =HTFflow[1].T_a;
      T_out =HTFflow[end].T_b;
      T_first = heatCapacitor[1].T;
      T_last = heatCapacitor[end].T;

      m = C_tot/cp;
      V_solid = m/rho;
      V_tot = Modelica.Constants.pi*do^2/4*N_pipes*L;
      UA = storageHeatTransfer.y*Modelica.Constants.pi*di*L*N_pipes;
      A_tot = Modelica.Constants.pi*do^2/4*N_pipes;
      y=di^2/do^2;
      der(E_TES) = Q_tot;

      //connectors
      for i in 1:N-1 loop
        connect(HTFflow[i].flowPort_b, HTFflow[i + 1].flowPort_a);
      end for;

      for i in 1:N loop
        connect(gain.y, heatTransfer[i].Gc);
        connect(heatCapacitor[i].port, heatTransfer[i].solid);
        connect(heatTransfer[i].fluid, HTFflow[i].heatPort);
      end for;

      connect(storageHeatTransfer.flowPort_a, flowPort_a) annotation (Line(points={{-42,0},
              {-52,0},{-52,-60},{-100,-60}},                                                          color={255,0,0}));
      connect(storageHeatTransfer.y, gain.u) annotation (Line(points={{-32,11},
              {-32,38},{-26,38}},                                                                  color={0,0,127}));
      connect(storageHeatTransfer.flowPort_b, HTFflow[1].flowPort_a)
        annotation (Line(points={{-22,0},{18,0}},  color={255,0,0}));
      connect(HTFflow[end].flowPort_b, flowPort_b)
        annotation (Line(points={{38,0},{52,0},{52,-60},{100,-60}},color={255,0,0}));
      annotation (Icon(coordinateSystem(preserveAspectRatio=false), graphics={
            Rectangle(
              extent={{-100,90},{100,-10}},
              lineColor={95,95,95},
              fillColor={215,215,215},
              fillPattern=FillPattern.Sphere),
              Rectangle(
              extent={{-100,-30},{100,-90}},
              lineColor={28,108,200},
              fillColor={255,255,255},
              fillPattern=FillPattern.HorizontalCylinder),
          Text(
            extent={{-100,90},{100,-10}},
            lineColor={0,0,0},
            fillPattern=FillPattern.HorizontalCylinder,
            fillColor={255,255,255},
              textString="thermal
storage"),  Line(points={{-20,-90},{-20,-30}},
                                             color={255,255,255}),
            Line(points={{20,-90},{20,-30}},
                                           color={255,255,255}),
            Line(points={{60,-90},{60,-30}},
                                           color={255,255,255}),
            Rectangle(
              extent={{-100,-10},{100,-30}},
              lineColor={0,0,0},
              fillColor={215,215,215},
              fillPattern=FillPattern.Forward),
            Rectangle(
              extent={{-100,-10},{100,-14}},
              lineColor={0,0,0},
              fillColor={135,135,135},
              fillPattern=FillPattern.Solid),
            Rectangle(
              extent={{-100,-26},{100,-30}},
              lineColor={0,0,0},
              fillColor={135,135,135},
              fillPattern=FillPattern.Solid),
            Line(points={{-60,-90},{-60,-30}},
                                             color={255,255,255})}), Diagram(
            coordinateSystem(preserveAspectRatio=false), graphics={
              Line(points={{28,48},{28,60}},
                                           color={255,0,0}),
              Line(points={{28,10},{28,28}},
                                           color={255,0,0}),
              Line(points={{-4,38},{18,38}},
                                           color={0,0,0})}));
    end TES;

    model HeatExchanger

      parameter Modelica.Thermal.FluidHeatFlow.Media.Medium hotMedium=Modelica.Thermal.FluidHeatFlow.Media.Medium()
        "Medium in hot side" annotation(choicesAllMatching=true);

      parameter Modelica.Thermal.FluidHeatFlow.Media.Medium coldMedium=Modelica.Thermal.FluidHeatFlow.Media.Medium()
        "Medium in cold side" annotation(choicesAllMatching=true);

      parameter Modelica.SIunits.Temperature        T0_hot, T0_cold;
      parameter Integer                             N(min=1) = 2;
      parameter Modelica.SIunits.Mass               m_hot, m_cold;
      //parameter Modelica.SIunits.ThermalResistance  R "Total thermal resistance";
      parameter Modelica.SIunits.ThermalConductance UA;
      parameter Boolean measureTcoldout=true "Measure the cold temperature outlet" annotation(Evaluate=true);

      Modelica.SIunits.HeatFlowRate                 Q_tot;
      Modelica.SIunits.Temperature                  T_hot_in, T_hot_out, T_cold_in, T_cold_out;
      Modelica.SIunits.Energy                       E_HX;

      Modelica.Thermal.FluidHeatFlow.Interfaces.FlowPort_a hotPort_a(medium=hotMedium)
        annotation (Placement(transformation(extent={{90,10},{110,30}}),
            iconTransformation(extent={{90,10},{110,30}})));
      Modelica.Thermal.FluidHeatFlow.Interfaces.FlowPort_b hotPort_b(medium=hotMedium)
        annotation (Placement(transformation(extent={{-110,10},{-90,30}}),
            iconTransformation(extent={{-110,10},{-90,30}})));
      Modelica.Thermal.FluidHeatFlow.Interfaces.FlowPort_a coldPort_a(medium=coldMedium)
        annotation (Placement(transformation(extent={{-110,-50},{-90,-30}}),
            iconTransformation(extent={{-110,-50},{-90,-30}})));
      Modelica.Thermal.FluidHeatFlow.Interfaces.FlowPort_b coldPort_b(medium=coldMedium)
        annotation (Placement(transformation(extent={{90,-50},{110,-30}}),
            iconTransformation(extent={{90,-50},{110,-30}})));
      Modelica.Thermal.FluidHeatFlow.Components.HeatedPipe coldPipe[N](each medium=coldMedium,
        each T0=T0_cold,
        each V_flowLaminar=1,
        each V_flowNominal=2,
        each h_g=0,
        each m=m_cold/N,
        each T0fixed=false,
        each dpLaminar=100,
        each dpNominal=2000)
        annotation (Placement(transformation(extent={{-10,-30},{10,-50}})));
      Modelica.Thermal.FluidHeatFlow.Components.HeatedPipe hotPipe[N](each medium=hotMedium,
        each T0=T0_hot,
        each V_flowLaminar=1,
        each V_flowNominal=2,
        each h_g=0,
        each m=m_hot/N,
        each T0fixed=false,
        each dpLaminar=100,
        each dpNominal=2000)
                         annotation (
          Placement(transformation(
            extent={{-10,-10},{10,10}},
            rotation=0,
            origin={0,40})));
      Modelica.Thermal.HeatTransfer.Components.ThermalResistor thermalResistor[N](each R=N/UA)
        annotation (Placement(transformation(
            extent={{-10,-10},{10,10}},
            rotation=90,
            origin={0,6})));
      Modelica.Blocks.Interfaces.RealOutput Q_HX annotation (Placement(transformation(
              extent={{-10,-10},{10,10}},
            rotation=270,
            origin={0,-70}),               iconTransformation(extent={{-10,-10},{10,
                10}},
            rotation=270,
            origin={-20,-80})));

      Modelica.Blocks.Interfaces.RealOutput T_hotin annotation (Placement(
            transformation(
            extent={{-10,-10},{10,10}},
            rotation=270,
            origin={40,-70}), iconTransformation(
            extent={{-10,-10},{10,10}},
            rotation=270,
            origin={20,-80})));
      Modelica.Blocks.Interfaces.RealOutput T_hotout annotation (Placement(
            transformation(
            extent={{-10,-10},{10,10}},
            rotation=270,
            origin={-40,-70}), iconTransformation(
            extent={{-10,-10},{10,10}},
            rotation=270,
            origin={-60,-80})));
      Modelica.Blocks.Interfaces.RealOutput T_coldout annotation (Placement(
          transformation(
            extent={{-10,-10},{10,10}},
            rotation=270,
            origin={70,-70}),
          iconTransformation(
            extent={{-10,-10},{10,10}},
            rotation=270,
            origin={60,-80}),
          visible=DynamicSelect(true, measureTcoldout)));
    equation

      //Q_tot = sum(thermalResistor[1:N].port_b.Q_flow);
      Q_tot = sum(coldPipe[1:N].heatPort.Q_flow);
      T_hot_in = hotPipe[end].T_b;
      T_hot_out = hotPipe[1].T_a;
      T_cold_in = coldPipe[1].T_a;
      T_cold_out = coldPipe[end].T_b;
      Q_HX = Q_tot;
      T_hotin = T_hot_in;
      T_hotout = T_hot_out;
      T_coldout = T_cold_out;
      der(E_HX) = Q_tot;

      connect(coldPort_a, coldPipe[1].flowPort_a)
        annotation (Line(points={{-100,-40},{-10,-40}},color={255,0,0}));
      connect(coldPipe[N].flowPort_b, coldPort_b)
        annotation (Line(points={{10,-40},{54,-40},{54,-40},{100,-40}},
                                                     color={255,0,0}));
      connect(hotPort_a, hotPipe[N].flowPort_b)
        annotation (Line(points={{100,20},{56,20},{56,40},{10,40}},
                                                   color={255,0,0}));
      connect(hotPort_b, hotPipe[1].flowPort_a)
        annotation (Line(points={{-100,20},{-54,20},{-54,40},{-10,40}},
                                                     color={255,0,0}));
      for i in 1:N-1 loop
        connect(coldPipe[i].flowPort_b, coldPipe[i+1].flowPort_a);
        connect(hotPipe[i].flowPort_b, hotPipe[i+1].flowPort_a);
      end for;

      for i in 1:N loop
        connect(coldPipe[i].heatPort, thermalResistor[i].port_a);
        connect(thermalResistor[i].port_b, hotPipe[i].heatPort);
      end for;

      annotation (Icon(coordinateSystem(preserveAspectRatio=false), graphics={
              Rectangle(
              extent={{-100,-10},{100,-70}},
              lineColor={28,108,200},
              fillColor={170,255,255},
              fillPattern=FillPattern.HorizontalCylinder),
                                              Rectangle(extent={{-100,50},{100,-10}},
                lineColor={28,108,200},
              fillColor={238,46,47},
              fillPattern=FillPattern.HorizontalCylinder),
            Text(
              extent={{-80,110},{80,50}},
              lineColor={0,0,0},
              fillPattern=FillPattern.HorizontalCylinder,
              fillColor={170,255,255},
              textString="%name"),
            Text(
              extent={{-60,50},{60,-10}},
              lineColor={0,0,0},
              fillPattern=FillPattern.HorizontalCylinder,
              fillColor={170,255,255},
              textString="Hot"),
            Text(
              extent={{-60,-10},{60,-70}},
              lineColor={0,0,0},
              fillPattern=FillPattern.HorizontalCylinder,
              fillColor={170,255,255},
              textString="Cold")}),                                  Diagram(
            coordinateSystem(preserveAspectRatio=false)));
    end HeatExchanger;

    model AirControl
      "Airflow varies depending on if moulds are above or below the solodification point"

      extends Modelica.Blocks.Interfaces.SO;

      parameter Modelica.SIunits.Temperature    target = 273.15+1205 "high air flow when T>target, low airflow otherwise";
      parameter Modelica.SIunits.VolumeFlowRate aboveV_flow;
      parameter Modelica.SIunits.VolumeFlowRate belowV_flow;
      parameter Real width = 90;
      parameter Real period = 7200;

      Boolean isBelow;

    protected
      Modelica.SIunits.VolumeFlowRate V_flow;

    public
      Modelica.Blocks.Interfaces.RealInput firstMould
        annotation (Placement(transformation(extent={{-140,20},{-100,60}})));
      Modelica.Blocks.Interfaces.RealInput lastMould
        annotation (Placement(transformation(extent={{-140,-60},{-100,-20}})));

      Modelica.Blocks.Sources.BooleanPulse booleanPulse(
        width=width,
        period=period,
        startTime=width*period/100)
        annotation (Placement(transformation(extent={{-16,-10},{4,10}})));
    initial equation
      pre(isBelow) =if firstMould > target then false else true;
      pre(V_flow) =if firstMould > target then aboveV_flow else belowV_flow;
    equation
      when lastMould < target then
        isBelow=true;
        V_flow=belowV_flow;
      elsewhen firstMould > target then
        isBelow=false;
        V_flow=aboveV_flow;
      elsewhen booleanPulse.y then
        V_flow = aboveV_flow;
        isBelow = pre(isBelow);
      end when;

      y=V_flow;

       annotation (Icon(coordinateSystem(preserveAspectRatio=false), graphics={
            Polygon(
              points={{90,-70},{68,-62},{68,-78},{90,-70}},
              lineColor={192,192,192},
              fillColor={192,192,192},
              fillPattern=FillPattern.Solid),
            Line(points={{-90,-70},{82,-70}}, color={192,192,192}),
            Line(points={{79,-70},{39,-70},{39,44},{-1,44},{-1,-70},{-41,-70},{-41,44},
                  {-80,44}}),
            Line(points={{-80,68},{-80,-80}}, color={192,192,192}),
            Polygon(
              points={{-80,90},{-88,68},{-72,68},{-80,90}},
              lineColor={192,192,192},
              fillColor={192,192,192},
              fillPattern=FillPattern.Solid)}),                   Diagram(
            coordinateSystem(preserveAspectRatio=false)));
    end AirControl;

    block ViewFactor
      parameter Real amplitude=0.5 "Amplitude of trapezoid";
      parameter Modelica.SIunits.Time pouring(final min=0) = 1800
        "Pouring time";
      parameter Real active(final min=0, final max=100) = 90
        "active in percentage";
      parameter Modelica.SIunits.Time period(final min=Modelica.Constants.small, start=1)=7200
        "Time for one period";
      parameter Integer nperiod=-1
        "Number of periods (< 0 means infinite number of periods)";
      parameter Real offset=0.5 "Offset of output signal";
      parameter Modelica.SIunits.Time startTime(final max=pouring)=0 "Output = offset for time < startTime";
      extends Modelica.Blocks.Interfaces.SO;
      Real dy;
      Modelica.SIunits.Time T_ref "reference pouint for dy";
    protected
      parameter Modelica.SIunits.Time T_active=period*active/100
        "End time of active phase in one period";
      parameter Modelica.SIunits.Time T_startconstant = pouring-startTime;
      parameter Modelica.SIunits.Time T_stopconstant = T_active-startTime;
      Modelica.SIunits.Time T_start "Start time of current period";
      Integer count "Period count";
      Real x;

    initial algorithm
      count := integer((time - startTime)/period);
      T_start := startTime + count*period;
      T_ref := startTime + count*period;
      y :=offset;
    equation
      dy = amplitude/(pouring/2);

      when integer((time - startTime)/period) > pre(count) then
        count = pre(count) + 1;
        T_start = time;
    //    T_ref = time;
      end when;

      when {y>=offset+amplitude, y<=offset+amplitude, integer((time - startTime)/period) > pre(count)} then  //or y<offset+amplitude
        T_ref = time;
      end when;

      x = if (time>=T_start+T_startconstant and time <=T_start+T_stopconstant) then 0 else 1;

      der(y) = (
       if time < startTime then
         0
       else
         if (time < (T_start+T_active)) then
           if time>=T_start+T_startconstant and time <=T_start+T_stopconstant then
             0
           elseif (T_ref<=T_start) then
             dy
           else
              -dy
         else
             0);

      annotation (Icon(coordinateSystem(preserveAspectRatio=false), graphics={
            Line(points={{-80,68},{-80,-80}}, color={192,192,192}),
            Polygon(
              points={{-80,90},{-88,68},{-72,68},{-80,90}},
              lineColor={192,192,192},
              fillColor={192,192,192},
              fillPattern=FillPattern.Solid),
            Line(points={{-90,-70},{82,-70}}, color={192,192,192}),
            Polygon(
              points={{90,-70},{68,-62},{68,-78},{90,-70}},
              lineColor={192,192,192},
              fillColor={192,192,192},
              fillPattern=FillPattern.Solid),
            Text(
              extent={{-147,-152},{153,-112}},
              lineColor={0,0,0},
              textString="period=%period"),
            Line(points={{-80,-70},{-56,-70},{-38,-18},{18,-18},{24,-6},{24,-6},
                  {42,-70}}, color={0,0,0})}),                       Diagram(
            coordinateSystem(preserveAspectRatio=false)));
    end ViewFactor;

    model PowerCycleControl
      "Airflow varies depending on if moulds are above or below the solodification point"

      extends Modelica.Blocks.Interfaces.SO;

      parameter Modelica.SIunits.Temperature    target = 273.15+100 "high air flow when T>target, low airflow otherwise";
      parameter Modelica.SIunits.VolumeFlowRate aboveV_flow;
      parameter Modelica.SIunits.VolumeFlowRate belowV_flow;

    protected
      Modelica.SIunits.VolumeFlowRate V_flow;

    public
      Modelica.Blocks.Interfaces.RealInput T_HX_in
        annotation (Placement(transformation(extent={{-140,-20},{-100,20}})));

    initial equation
      pre(V_flow) =if T_HX_in > target then aboveV_flow else belowV_flow;
    equation
      when T_HX_in > target then
        V_flow=aboveV_flow;
      elsewhen
        T_HX_in<target then
        V_flow=belowV_flow;
      end when;

      y=V_flow;

       annotation (Icon(coordinateSystem(preserveAspectRatio=false), graphics={
            Polygon(
              points={{90,-70},{68,-62},{68,-78},{90,-70}},
              lineColor={192,192,192},
              fillColor={192,192,192},
              fillPattern=FillPattern.Solid),
            Line(points={{-90,-70},{82,-70}}, color={192,192,192}),
            Line(points={{79,-70},{39,-70},{39,44},{-1,44},{-1,-70},{-41,-70},{-41,44},
                  {-80,44}}),
            Line(points={{-80,68},{-80,-80}}, color={192,192,192}),
            Polygon(
              points={{-80,90},{-88,68},{-72,68},{-80,90}},
              lineColor={192,192,192},
              fillColor={192,192,192},
              fillPattern=FillPattern.Solid)}),                   Diagram(
            coordinateSystem(preserveAspectRatio=false)));
    end PowerCycleControl;

    model CarnotPower
      parameter Real             fraction(min=0,max=1)=0.5;
      parameter Modelica.SIunits.Temperature T_amb;

      Modelica.SIunits.Power     W_carnot,W_produced;
      Modelica.SIunits.Energy    E_carnot, E_produced;
      Real                       eta_carnot;

      Modelica.Blocks.Interfaces.RealInput Q annotation (Placement(
            transformation(
            extent={{-20,-20},{20,20}},
            rotation=90,
            origin={0,0}),   iconTransformation(
            extent={{-20,-20},{20,20}},
            rotation=90,
            origin={0,60})));
      Modelica.Blocks.Interfaces.RealInput T_hotout annotation (Placement(
            transformation(
            extent={{-20,-20},{20,20}},
            rotation=90,
            origin={-60,0}), iconTransformation(
            extent={{-20,-20},{20,20}},
            rotation=90,
            origin={-60,60})));

      Modelica.Blocks.Interfaces.RealInput T_hotin annotation (Placement(
            transformation(
            extent={{-20,-20},{20,20}},
            rotation=90,
            origin={60,0}), iconTransformation(
            extent={{-20,-20},{20,20}},
            rotation=90,
            origin={60,60})));
      Modelica.Blocks.Interfaces.RealOutput W_prod annotation (Placement(
            transformation(extent={{-80,-20},{-100,0}}), iconTransformation(extent={
                {-80,-30},{-120,10}})));
    equation
      if noEvent(T_hotin > T_hotout) then
        eta_carnot = 1-T_amb/(T_hotin-T_hotout)*log(T_hotin/T_hotout);
      elseif noEvent(T_hotin < T_hotout) then
        eta_carnot = 1-T_amb/(T_hotout-T_hotin)*log(T_hotout/T_hotin);
      else
        eta_carnot = 0;
      end if;

      W_carnot = Q*eta_carnot;
      W_produced = fraction*W_carnot;
      W_prod = W_produced;

      der(E_carnot) = W_carnot;
      der(E_produced) = W_produced;
      annotation (Icon(coordinateSystem(preserveAspectRatio=false), graphics={
              Rectangle(
              extent={{-80,40},{80,-60}},
              lineColor={0,0,0},
              fillColor={175,175,175},
              fillPattern=FillPattern.Solid), Text(
              extent={{-78,38},{78,-58}},
              lineColor={0,0,0},
              fillColor={175,175,175},
              fillPattern=FillPattern.Solid,
              textString="Power")}),                                 Diagram(
            coordinateSystem(preserveAspectRatio=false)));
    end CarnotPower;
  end Components;

  model System
    extends Modelica.Icons.Example;

  public
    parameter Modelica.Thermal.FluidHeatFlow.Media.Medium HTF=Modelica.Thermal.FluidHeatFlow.Media.Water(rho = 55.13, cp = 1156, cv = 934, nue = 5.39e-7, lamda = 0.0467);// =0.075)  "Heat loop HTF; basert på trykksatt CO2, 70 bar 400'C"
    parameter Modelica.Thermal.FluidHeatFlow.Media.Medium air=Modelica.Thermal.FluidHeatFlow.Media.Air_70degC(rho = 0.736, cp = 1027, cv = 739, nue = 3.5e-5, lamda = 0.0378) "tunnel air flow, basert på 1 bar, 200'C luft"
      annotation(choicesAllMatching=true);
    parameter Modelica.Thermal.FluidHeatFlow.Media.Medium WF=Modelica.Thermal.FluidHeatFlow.Media.Water(rho = 6.30, cp = 2245, cv = 1659, nue = 3.17e-6, lamda = 0.0484)  "Power cycle WF, 16 bar damp ved 300'C"
      annotation(choicesAllMatching=true);
    parameter Components.Silicon.SiliconParameters silicon = Components.Silicon.SiliconParameters() "silicon parameters";

    //moulds and tunnel
    parameter Modelica.SIunits.Length height = 0.75;
    parameter Modelica.SIunits.Length width =  1.8;//1.8*1.215;
    parameter Modelica.SIunits.Length length = area/width;// 2.18*10;//2.0*10;
    final parameter Modelica.SIunits.Area area = 2*2.18*10*1.003;

    parameter Modelica.SIunits.Velocity v_air = 1;
    parameter Modelica.SIunits.MassFlowRate m_htf = 15;
    parameter Modelica.SIunits.CoefficientOfHeatTransfer h_free=10;

    parameter Real  active(min=0, max=100) = 75;
    parameter Real  period(min=0) = 7200;
    parameter Real  pouring(min=0,max=period) = 1800;

    parameter Integer N_moulds = 10;
    parameter Integer N_partitions(min=1) = 3;
    parameter Integer N_mouldlayers(min=1)= 15;
    parameter Modelica.SIunits.Mass m_silicon = 9000;

    parameter Real e1 = 0.55 "emissivity of molten cooling silicon";
    parameter Real e2 = 0.8 "emissivity of tunnel wall";
    parameter Real e3 = 0.99 "emissivity of surroundings, ~backbody";

    parameter Modelica.SIunits.Temperature T0_HTF = 273.15+280;
    parameter Modelica.SIunits.Temperature T_amb = 273.15+25;
    parameter Modelica.SIunits.Temperature T0_mould = 273.15+1450;

    parameter Modelica.SIunits.ThermalConductivity k_ins = 0.055;
    parameter Modelica.SIunits.Length L_ins = 0.01;

    //TES
    parameter Modelica.SIunits.Temperature T0_TESfirst = 353.4+273.15;
    parameter Modelica.SIunits.Temperature T0_TESlast = 384.3+273.15;

    parameter Integer N_TES = 10;
    parameter Integer N_TES_pipes = 350;//620;

    parameter Modelica.SIunits.HeatCapacity C_tot = 7e7;
    parameter Modelica.SIunits.Length   D = 80e-3;
    parameter Modelica.SIunits.Diameter di = 10e-3;

    //HX
    parameter Integer N_HX = 10;
    parameter Modelica.SIunits.ThermalConductance UA_HX = 1.1e4;
    parameter Modelica.SIunits.MassFlowRate m_WF = 3.2;
    parameter Modelica.SIunits.Temperature T0_WF = 100+273.15;

    Components.TunnelDiscrete tunnelDiscrete(
      air=air,
      HTF=HTF,
      N_moulds=N_moulds,
      N_partitions=N_partitions,
      N_mouldlayers=N_mouldlayers,
      active=active,
      period=period,
      pouring=pouring,
      e1=e1,
      e2=e2,
      e3=e3,
      T_amb=T_amb,
      T0_mould=T0_mould,
      T0_HTF=T0_HTF,
      length=length,
      width=width,
      height=height,
      m_silicon=m_silicon,
      v_air=v_air,
      s=silicon,
      h_free=h_free,
      useWallLoss=true,
      k_ins=k_ins,
      L_ins=L_ins)
      annotation (Placement(transformation(extent={{-18,52},{22,92}})));
    Components.TES tES(medium=HTF,
      T0_HTF=T0_HTF,
      N=N_TES,
      N_pipes=N_TES_pipes,
      C_tot=C_tot,
      D=D,
      di=di,
      T0_first=T0_TESfirst,
      T0_last=T0_TESlast)          annotation (Placement(transformation(extent={{76,0},{
              36,40}})));
    Components.HeatExchanger             heatExchanger(
      T0_hot=T0_HTF,
      T0_cold=T0_WF,
      N=N_HX,
      hotMedium=HTF,
      coldMedium=WF,
      UA=UA_HX,
      m_hot=45,
      m_cold=45,
      measureTcoldout=true)
      annotation (Placement(transformation(extent={{-18,-16},{18,24}})));
    Components.CarnotPower carnotPower(T_amb=T_amb)
      annotation (Placement(transformation(extent={{-16,-42},{8,-20}})));
    Modelica.Thermal.FluidHeatFlow.Sources.VolumeFlow volumeFlow(medium=HTF,
      m=0,
      useVolumeFlowInput=false,
      constantVolumeFlow=m_htf/HTF.rho)
      annotation (Placement(transformation(extent={{-58,70},{-38,90}})));
    Modelica.Thermal.FluidHeatFlow.Sources.AbsolutePressure referencePressure(medium=
          HTF, p=100000) annotation (Placement(transformation(
          extent={{-10,-10},{10,10}},
          rotation=270,
          origin={66,62})));
    Modelica.Thermal.FluidHeatFlow.Sources.VolumeFlow volumeFlow1(
      medium=WF,
      m=0,
      useVolumeFlowInput=false,
      constantVolumeFlow=m_WF/WF.rho)
      annotation (Placement(transformation(extent={{-50,2},{-30,-18}})));
    Modelica.Thermal.FluidHeatFlow.Sources.Ambient RC_1(
      medium=WF,
      constantAmbientTemperature=T0_WF,
      constantAmbientPressure=100000)
      annotation (Placement(transformation(extent={{-62,-14},{-74,-2}})));
    Modelica.Thermal.FluidHeatFlow.Sources.Ambient RC_2(
      medium=WF,
      constantAmbientTemperature=T_amb,
      constantAmbientPressure=100000)
      annotation (Placement(transformation(extent={{48,-26},{60,-14}})));
  equation

    connect(tunnelDiscrete.flowPort_b, tES.flowPort_a)
      annotation (Line(points={{18.6667,80},{92,80},{92,8},{76,8}},  color={255,0,0}));
    connect(tES.flowPort_b, heatExchanger.hotPort_a)
      annotation (Line(points={{36,8},{18,8}},                                color={255,0,0}));
    connect(volumeFlow.flowPort_b, tunnelDiscrete.flowPort_a)
      annotation (Line(points={{-38,80},{-14.6667,80}},
                                                   color={255,0,0}));
    connect(referencePressure.flowPort, tES.flowPort_a) annotation (Line(points={{66,72},
            {66,80},{92,80},{92,8},{76,8}},            color={255,0,0}));
    connect(carnotPower.T_hotout, heatExchanger.T_hotout)
      annotation (Line(points={{-11.2,-24.4},{-11.2,-12},{-10.8,-12}},
                                                                    color={0,0,127}));
    connect(carnotPower.Q, heatExchanger.Q_HX)
      annotation (Line(points={{-4,-24.4},{-4,-12},{-3.6,-12}},
                                                             color={0,0,127}));
    connect(heatExchanger.T_hotin, carnotPower.T_hotin)
      annotation (Line(points={{3.6,-12},{3.6,-24.4},{3.2,-24.4}}, color={0,0,127}));
    connect(volumeFlow1.flowPort_b, heatExchanger.coldPort_a)
      annotation (Line(points={{-30,-8},{-24,-8},{-24,-4},{-18,-4}},     color={255,0,0}));
    connect(heatExchanger.coldPort_b, RC_2.flowPort) annotation (Line(points={{18,-4},
            {36,-4},{36,-20},{48,-20}},       color={255,0,0}));
    connect(RC_1.flowPort, volumeFlow1.flowPort_a)
      annotation (Line(points={{-62,-8},{-50,-8}},   color={255,0,0}));
    connect(heatExchanger.hotPort_b, volumeFlow.flowPort_a) annotation (Line(
          points={{-18,8},{-78,8},{-78,80},{-58,80}}, color={255,0,0}));
      annotation(choicesAllMatching=true,
                Icon(coordinateSystem(preserveAspectRatio=false, extent={{-140,-100},
              {100,100}})),                                        Diagram(coordinateSystem(preserveAspectRatio=false, extent={
              {-140,-100},{100,100}})));
  end System;

  model SystemTunnelOpenings
    extends System( tunnelDiscrete(useVolumeFlowInput=true, measureSolidification=
           true),
          T0_TESfirst = 371+273.15,
          T0_TESlast = 406.8+273.15);
    Components.AirControl airControl(aboveV_flow=height*width*v_air, belowV_flow=0,
      target=silicon.T_solidus,
      width=active,
      period=period)
      annotation (Placement(transformation(extent={{-44,44},{-30,58}})));
  equation
    connect(airControl.y, tunnelDiscrete.airVolumeFlow) annotation (Line(points={{-29.3,
            51},{-24,51},{-24,68},{-16.3333,68}},       color={0,0,127}));
    connect(tunnelDiscrete.lastMouldTemp, airControl.lastMould) annotation (Line(
          points={{-11.6667,58},{-11.6667,40},{-50,40},{-50,48.2},{-45.4,48.2}},
          color={0,0,127}));
    connect(tunnelDiscrete.firstMouldTemp, airControl.firstMould) annotation (
        Line(points={{16,58},{16,36},{-54,36},{-54,53.8},{-45.4,53.8}}, color={0,0,
            127}));
  end SystemTunnelOpenings;

  model System4TunnelOpeningsPItemp
    parameter Modelica.SIunits.Temperature   targetTemp= 273.15+310;
    parameter Real                           k = -1e-3;
    parameter Real                           Ti = 3;
    parameter Modelica.SIunits.MassFlowRate  m_wfmin = 1;
    parameter Modelica.SIunits.MassFlowRate  m_wfmax= 5;

    extends System( tunnelDiscrete(useVolumeFlowInput=true, measureSolidification=
           true),
          T0_TESfirst = 358.8+273.15,
          T0_TESlast = 397.5+273.15,
          volumeFlow1(useVolumeFlowInput=true));
    Components.AirControl airControl(aboveV_flow=height*width*v_air, belowV_flow=0,
      target=silicon.T_solidus,
      width=active,
      period=period)
      annotation (Placement(transformation(extent={{-44,44},{-30,58}})));
    Modelica.Blocks.Continuous.LimPID PID(
      controllerType=Modelica.Blocks.Types.SimpleController.PI,
      limitsAtInit=true,
      y_start=m_WF,
      yMax=m_wfmax,
      yMin=m_wfmin,
      initType=Modelica.Blocks.Types.InitPID.InitialOutput,
      k=k,
      Ti=Ti)
      annotation (Placement(transformation(extent={{-94,-38},{-82,-26}})));
    Modelica.Blocks.Math.Gain gain(k=1/WF.rho)
      annotation (Placement(transformation(extent={{-64,-38},{-52,-26}})));
    Modelica.Blocks.Sources.Constant const(k=targetTemp)
      annotation (Placement(transformation(extent={{-122,-38},{-110,-26}})));
  equation
    connect(airControl.y, tunnelDiscrete.airVolumeFlow) annotation (Line(points={{-29.3,
            51},{-24,51},{-24,68},{-16.3333,68}},       color={0,0,127}));
    connect(tunnelDiscrete.lastMouldTemp, airControl.lastMould) annotation (Line(
          points={{-11.6667,58},{-11.6667,40},{-50,40},{-50,48.2},{-45.4,48.2}},
          color={0,0,127}));
    connect(tunnelDiscrete.firstMouldTemp, airControl.firstMould) annotation (
        Line(points={{16,58},{16,36},{-54,36},{-54,53.8},{-45.4,53.8}}, color={0,0,
            127}));
    connect(const.y, PID.u_s)
      annotation (Line(points={{-109.4,-32},{-95.2,-32}}, color={0,0,127}));
    connect(PID.y, gain.u)
      annotation (Line(points={{-81.4,-32},{-65.2,-32}}, color={0,0,127}));
    connect(gain.y, volumeFlow1.volumeFlow) annotation (Line(points={{-51.4,-32},{
            -40,-32},{-40,-18}}, color={0,0,127}));
    connect(PID.u_m, carnotPower1.T_coldout) annotation (Line(points={{-88,-39.2},
            {-88,-48},{10,-48},{10,-57},{-12,-57},{-12,-79}}, color={0,0,127}));
  end System4TunnelOpeningsPItemp;

  model System4TunnelOpeningsPIpower

    parameter Modelica.SIunits.Power         W = 400e3;
    parameter Real                           k = 5e-6;
    parameter Real                           Ti = 3;
    parameter Modelica.SIunits.MassFlowRate  m_wfmin = 1;
    parameter Modelica.SIunits.MassFlowRate  m_wfmax = 5;

    extends System( tunnelDiscrete(useVolumeFlowInput=true, measureSolidification=
           true),
          T0_TESfirst = 346.5+273.15,
          T0_TESlast = 389+273.15,
          volumeFlow1(useVolumeFlowInput=true));
    Components.AirControl airControl(aboveV_flow=height*width*v_air, belowV_flow=0,
      target=silicon.T_solidus,
      width=active,
      period=period)
      annotation (Placement(transformation(extent={{-44,44},{-30,58}})));
    Modelica.Blocks.Continuous.LimPID PID(
      limitsAtInit=true,
      initType=Modelica.Blocks.Types.InitPID.InitialOutput,
      controllerType=Modelica.Blocks.Types.SimpleController.PI,
      k=k,
      Ti=Ti,
      yMax=m_wfmax,
      yMin=m_wfmin,
      y_start=m_WF)
      annotation (Placement(transformation(extent={{-6,-6},{6,6}},
          rotation=0,
          origin={-80,-32})));
    Modelica.Blocks.Math.Gain gain(k=1/WF.rho)
      annotation (Placement(transformation(extent={{-66,-42},{-46,-22}})));
    Modelica.Blocks.Sources.Constant const(k=W)
      annotation (Placement(transformation(extent={{-116,-42},{-96,-22}})));
  equation
    connect(airControl.y, tunnelDiscrete.airVolumeFlow) annotation (Line(points={{-29.3,
            51},{-24,51},{-24,68},{-16.3333,68}},       color={0,0,127}));
    connect(tunnelDiscrete.lastMouldTemp, airControl.lastMould) annotation (Line(
          points={{-11.6667,58},{-11.6667,40},{-50,40},{-50,48.2},{-45.4,48.2}},
          color={0,0,127}));
    connect(tunnelDiscrete.firstMouldTemp, airControl.firstMould) annotation (
        Line(points={{16,58},{16,36},{-54,36},{-54,53.8},{-45.4,53.8}}, color={0,0,
            127}));
    connect(carnotPower.W_prod, PID.u_m) annotation (Line(points={{-16,-32.1},{-26,
            -32.1},{-26,-68},{-80,-68},{-80,-39.2}}, color={0,0,127}));
    connect(const.y, PID.u_s)
      annotation (Line(points={{-95,-32},{-87.2,-32}}, color={0,0,127}));
    connect(PID.y, gain.u)
      annotation (Line(points={{-73.4,-32},{-68,-32}}, color={0,0,127}));
    connect(gain.y, volumeFlow1.volumeFlow)
      annotation (Line(points={{-45,-32},{-40,-32},{-40,-18}}, color={0,0,127}));
  end System4TunnelOpeningsPIpower;
end MasterAppendix;
